let fs = require('fs');
let path = require('path');

function catchError(error) { }


function isPageError(code) {
  return code == '' || / jscontent="errorCode" jstcache="\d+"|diagnoseConnectionAndRefresh|dnserror_unavailable_header|id="reportCertificateErrorRetry"|400 Bad Request|403 Forbidden|404 Not Found|500 Internal Server Error|502 Bad Gateway|503 Service Temporarily Unavailable|504 Gateway Time-out/i.test(code);
}




function getRootPath(contextModule) {
  // or user process.cwd
  let rootPath = path.dirname(contextModule.id);
  while (rootPath.length && !fs.existsSync(path.resolve(rootPath, 'config.json'))) {
    rootPath = rootPath.substring(0, rootPath.lastIndexOf(path.sep));
  }

  return rootPath;
}

function escapeTestVars(testVars, str) {
  if (typeof str === 'string') {
    return str.replace(/\{\{(.+?)\}\}/g, function (all, key) {
      return testVars[key] || '';
    });
  }
  else {
    return str;
  }
}

function callSpec(rootPath, name) {
  try {
    require(rootPath + '/' + name)();
  }
  catch (e) {
    console.log(e)
    process.exit(1);
  }
}



function appendToContext(mocha, content) {
  try {
    const test = mocha.currentTest || mocha.test;

    if (!test.context) {
      test.context = content;
    } else if (Array.isArray(test.context)) {
      test.context.push(content);
    } else {
      test.context = [test.context];
      test.context.push(content);
    }
  } catch (e) {
    console.log('error', e);
  }
};


module.exports = function helper(contextModule) {
  const rootPath = getRootPath(contextModule);
  const config = require(path.resolve(rootPath, 'config.json'));
  return {
    rootPath,
    config,
    isPageError,
    catchError,
    callSpec: callSpec.bind(null, rootPath),
    _: escapeTestVars.bind(null, escapeTestVars),
  };
};
