

// auto inject code

const fs = require('fs');
const path = require('path');
const chai = require("chai");
const strip = require('strip-comments');
const JWebDriver = require('jwebdriver');
const resemble = require('resemblejs-node');
const specHelper = require('uirecorder2').specHelper;


chai.use(JWebDriver.chaiSupportChainPromise);
resemble.outputSettings({
  errorType: 'flatDifferenceIntensity'
});


// todo inject script (only eval __env__)



if (module.parent && /mocha\.js/.test(module.parent.id)) {
  runThisSpec();
}


function runThisSpec() {
  let helper = specHelper(module);
  let { rootPath, config } = helper;
  // read config
  let webdriver = process.env['webdriver'] || '';
  let proxy = process.env['wdproxy'] || '';
  let webdriverConfig = Object.assign({}, config.webdriver);
  let host = webdriverConfig.host;
  let port = webdriverConfig.port || 4444;
  let match = webdriver.match(/([^\:]+)(?:\:(\d+))?/);
  if (match) {
    host = match[1] || host;
    port = match[2] || port;
  }
  let testVars = config.vars;
  let browsers = webdriverConfig.browsers;
  browsers = browsers.replace(/^\s+|\s+$/g, '');
  delete webdriverConfig.host;
  delete webdriverConfig.port;
  delete webdriverConfig.browsers;

  // read hosts
  let hostsPath = rootPath + '/hosts';
  let hosts = '';
  if (fs.existsSync(hostsPath)) {
    hosts = fs.readFileSync(hostsPath).toString();
  }
  let specName = path.relative(rootPath, __filename).replace(/\\/g, '/').replace(/\.js$/, '');
  if (module.exports.desc) {
    specName = module.exports.desc;
  }
  browsers.split(/\s*,\s*/).forEach(function (browserName) {
    let caseName = specName + ' : ' + browserName;

    let browserInfo = browserName.split(' ');
    browserName = browserInfo[0];
    let browserVersion = browserInfo[1];

    describe(caseName, function () {

      this.timeout(600000);
      this.slow(1000);

      let driver;
      before(function () {
        let self = this;
        let driver = new JWebDriver({
          'host': host,
          'port': port,
          // 'logLevel': 2
        });
        let sessionConfig = Object.assign({}, webdriverConfig, {
          'browserName': browserName,
          'version': browserVersion,
          'ie.ensureCleanSession': true,
        });
        if (proxy) {
          sessionConfig.proxy = {
            'proxyType': 'manual',
            'httpProxy': proxy,
            'sslProxy': proxy
          }
        }
        else if (hosts) {
          sessionConfig.hosts = hosts;
        }

        try {
          self.driver = driver.session(sessionConfig).windowSize(1024, 768).config({
            // // ie gecko driver no support well ?
            // pageloadTimeout: 30000, // page onload timeout
            // scriptTimeout: 5000, // sync script timeout
            // asyncScriptTimeout: 10000 // async script timeout
          });
        } catch (e) {
          console.log(e);
        }

        self.testVars = testVars;
        let casePath = path.dirname(caseName);
        self.screenshotPath = rootPath + '/screenshots/' + casePath;
        self.diffbasePath = rootPath + '/diffbase/' + casePath;
        self.caseName = caseName.replace(/.*\//g, '').replace(/\s*[:\.\:\-\s]\s*/g, '_');
        mkdirs(self.screenshotPath);
        mkdirs(self.diffbasePath);
        self.stepId = 0;
        return self.driver;
      });

      before(function() {
        // gecko
        this.driver.sendKeys = this.driver.sendElementKeys;
        // todo inject script when page load

      });

      beforeEach(function () {
        let self = this;
        self.stepId++;
        if (self.skipAll) {
          self.skip();
        }
      });

      module.exports.case(helper);

      afterEach(async function () {
        let self = this;
        let currentTest = self.currentTest;
        let title = currentTest.title;
        if (currentTest.state === 'failed' && /^(url|waitBody|switchWindow|switchFrame):/.test(title)) {
          self.skipAll = true;
        }
        if (!/^(closeWindow):/.test(title)) {
          let filepath = self.screenshotPath + '/' + self.caseName + '_' + self.stepId;
          let driver = self.driver;
          try {
            // catch error when get alert msg
            await driver.getScreenshot(filepath + '.png');
            let url = await driver.url();
            let html = await driver.source();
            html = '<!--url: ' + url + ' -->\n' + html;
            fs.writeFileSync(filepath + '.html', html);
            // let cookies = await driver.cookies();
            // fs.writeFileSync(filepath + '.cookie', JSON.stringify(cookies));
            appendToContext(self, filepath + '.png');
          }
          catch (e) { }
        }
      });

      after(function () {
        return this.driver.close();
      });

    });
  });
}



function appendToContext(mocha, content) {
  try {
    const test = mocha.currentTest || mocha.test;

    if (!test.context) {
      test.context = content;
    } else if (Array.isArray(test.context)) {
      test.context.push(content);
    } else {
      test.context = [test.context];
      test.context.push(content);
    }
  } catch (e) {
    console.log('error', e);
  }
};

function mkdirs(dirname) {
  if (fs.existsSync(dirname)) {
    return true;
  } else {
    if (mkdirs(path.dirname(dirname))) {
      fs.mkdirSync(dirname);
      return true;
    }
  }
}
