window.__env__ = '执行自动化测试';

(function() {
    'use strict';
    console.log('ember test injector setup.');
    let Ember = window.Ember;
    let registry;
    const DATA_TEST_FLAG = 'data-test';

    if (window.Ember) {
      console.log('Ember is ready before injector setup.');
      initInject(Ember.Component);
    } else {
      Ember = window.Ember = {};
      let _Component_ = null;
      Object.defineProperty(Ember, 'Component', {
        set(c) {
          _Component_ = c;
          // init reopen Component
          initInject(c);
        },
        get() {
          return _Component_;
        }
      });
    }

    // Your code here...
    function initInject(Component) {
      Component.reopen({
        didInsertElement() {
          this._super(...arguments);
          let el = this.element;
          let testAttr = generateTestFlag(this);
          if (!el && this.elementId) {
            el = document.body.querySelector('#'+this.elementId);
          }
          if (el) {
            if (!el.getAttribute(DATA_TEST_FLAG)) {
              el.setAttribute(DATA_TEST_FLAG, testAttr);
              el.classList.add(`js-${testAttr}`); // use class instead attr
            }
          }
        }
      });
      let root = document.body.querySelector(':scope > .ember-view');
      if (root) {
        root.setAttribute(DATA_TEST_FLAG, 'root');
      }
    }

    function generateTestFlag(component) {
      let id = component.elementId;
      let view = component;
      let flag = 'default';
      if (component.constructor === Ember.LinkComponent) {
        flag = component.qualifiedRouteName;
      } else if (id) {
        if (!(/ember[\d]+$/i.test(id) || /\d+/.test(id))) {
          flag = id;
        } else {
          let moduleName;
          if (view.layout && view.layout.referrer) {
            moduleName = view.layout.referrer.moduleName;
          } else if(view.layout && view.layout.meta) {
            moduleName = view.layout.meta.moduleName;
          }
          if (moduleName) {
            let match = moduleName.match(/templates\/components\/(.*)\.hbs/);
            if (match && match[1]) {
              flag = match[1].replace(/\//g, '_');
            }
          }
        }
      }

      return flag;
    }

    function getOwner() {
      return Ember.Namespace.NAMESPACES[0].__container__.owner;
    }

    function getRegistry() {
      if (!registry) {
        registry = getOwner().lookup('-view-registry:main');
      }
      return registry;
    }

  })();


